@@ -21,6 +21,8 @@ from django.utils.dateparse import (
 
 import orjson
 
+from .utils import is_non_string_sequence
+
 
 try:
     from pydantic import BaseModel as PydanticBaseModel
@@ -35,6 +37,28 @@ class JSONDecodeError(Exception):
     pass
 
 
+class InvalidFieldNameError(Exception):
+    def __init__(self, field_name: str, data: Dict = None):
+        message = f"Cannot resolve '{field_name}'."
+
+        if data:
+            available = ", ".join(data.keys())
+            message = f"{message} Choices are: {available}"
+
+        super().__init__(message)
+
+
+class InvalidFieldAttributeError(Exception):
+    def __init__(self, field_name: str, field_attr: str, data: Dict = None):
+        message = f"Cannot resolve '{field_attr}'."
+
+        if data:
+            available = ", ".join(data[field_name].keys())
+            message = f"{message} Choices on '{field_name}' are: {available}"
+
+        super().__init__(message)
+
+
 def _parse_field_values_from_string(model: Model) -> None:
     """
     Convert the model fields' value to match the field type if appropriate.
@@ -75,16 +99,25 @@ def _get_model_dict(model: Model) -> dict:
 
     for field in model._meta.get_fields():
         if field.is_relation and field.many_to_many:
+            exclude_field_attributes = getattr(
+                model, "__unicorn__exclude_field_attribute", []
+            )
             related_name = field.name
 
             if field.auto_created:
                 related_name = field.related_name or f"{field.name}_set"
 
+            if related_name in exclude_field_attributes:
+                continue
+
             pks = []
 
             try:
                 related_descriptor = getattr(model, related_name)
-                pks = list(related_descriptor.values_list("pk", flat=True))
+
+                # Get `pk` from `all` because it will re-use the cached data if the m-2-m field is prefetched
+                # Using `values_list("pk", flat=True)` or `only()` won't use the cached prefetched values
+                pks = [m.pk for m in related_descriptor.all()]
             except ValueError:
                 # ValueError is throuwn when the model doesn't have an id already set
                 pass
@@ -217,6 +250,15 @@ def _exclude_field_attributes(
                 )
             elif len(field_splits) == 2:
                 (field_name, field_attr) = field_splits
+
+                if field_name not in dict_data:
+                    raise InvalidFieldNameError(field_name=field_name, data=dict_data)
+
+                if field_attr not in dict_data[field_name]:
+                    raise InvalidFieldAttributeError(
+                        field_name=field_name, field_attr=field_attr, data=dict_data
+                    )
+
                 del dict_data[field_name][field_attr]
 
 
@@ -238,6 +280,40 @@ def dumps(
     Returns a `str` instead of `bytes` (which deviates from `orjson.dumps`), but seems more useful.
     """
 
+    assert exclude_field_attributes is None or is_non_string_sequence(
+        exclude_field_attributes
+    ), "exclude_field_attributes type needs to be a sequence"
+
+    # This explicitly handles excluding many-to-many fields with a hacky private `__unicorn__exclude_field_attribute`
+    # attribute that gets used later in `_get_model_json`
+    if exclude_field_attributes:
+        updated_exclude_field_attributes = set()
+
+        for field_attributes in exclude_field_attributes:
+            field_attributes_split = field_attributes.split(".")
+            field_attribute = field_attributes_split[0]
+
+            for key in data.keys():
+                if isinstance(data[key], Model) and key == field_attribute:
+                    remaining_field_attributes = field_attributes_split[1:]
+
+                    if hasattr(data[key], "__unicorn__exclude_field_attribute"):
+                        data[key].__unicorn__exclude_field_attribute.append(
+                            remaining_field_attributes[0]
+                        )
+                    else:
+                        setattr(
+                            data[key],
+                            "__unicorn__exclude_field_attribute",
+                            remaining_field_attributes,
+                        )
+                    break
+                else:
+                    updated_exclude_field_attributes.add(field_attributes)
+
+        # Convert list to tuple again so it's hashable for `lru_cache`
+        exclude_field_attributes = tuple(updated_exclude_field_attributes)
+
     serialized_data = orjson.dumps(data, default=_json_serializer)
 
     if fix_floats:
